`timescale 1 ns / 100 ps
// ************************************************************
//-****************** The Task (Transaction command) *****
//localparam T_IDLE        = 0; // "T_IDLE       ";
//localparam RESET         = 1; // "RESET        ";
//localparam READ_SINGLE   = 2; // "READ_SINGLE  ";
//localparam READ_SINGLE2  = 3; // "READ_SINGLE  ";
//localparam READ_SEQ4     = 4; // "READ_SEQ4    ";
//localparam WRITE_SINGLE  = 5; // "WRITE_SINGLE ";
//localparam WRITE_1OFN    = 6; // "WRITE_1OFN   ";
//localparam WRITE_NOFM    = 7; // "WRITE_NOFM   ";
//localparam WRITE_LASTOFM = 8; // "WRITE_LASTOFM";
//localparam WRITE_WRAP4   = 9; // "WRITE_WRAP4  ";
//localparam WRITE_WRAP8   = 10; // "WRITE_WRAP8  ";

module client;
  event xactn; // transaction to server 
  event done;  // done with transaction, from server
  integer i;
  // `include "ahb_param.v"  
  `include "localparam.v"
  
  always 
	begin 
	  repeat (5) @ (posedge(server.hclk));
      // do reset
	   thetask.job = RESET;
      #1 -> xactn;   @done;
	  // do idle
	  thetask.job = T_IDLE;
      #1 -> xactn;   @done;
	  
	  // do a write
	  thetask.data=32'h00010000;
	  thetask.addr=32'h00000000;
      thetask.job = WRITE_SINGLE;
      #1 -> xactn;   @done;

	  // do a write2
	  thetask.data=32'h00010004;
	  thetask.addr=32'h00000004;
      thetask.job = WRITE_SINGLE;
      #1 -> xactn;   @done;
	  
	  
	  // do a read
	  thetask.addr=32'h00000000;
      thetask.job = READ_SINGLE;
      #1 -> xactn;   @done;

	   // do a read2
	  thetask.addr=32'h00000000;
      thetask.job = READ_SINGLE2;
      #1 -> xactn;   @done;

	  // other memory bank
	  // do a write
	  thetask.data=32'hFFFFF000;
	  thetask.addr=32'hFFFFF000;
      thetask.job = WRITE_SINGLE;
      #1 -> xactn;   @done;
	  
	  // do a read
	  thetask.addr=32'hFFFFF000;
      thetask.job = READ_SINGLE;
      #1 -> xactn;   @done;

	   // do idle
	  thetask.job = T_IDLE;
      #1 -> xactn;   @done;
	  
	  // wait for a while 
	  repeat (5) @ (posedge(server.hclk));

	  // do 4 writes
	  // The following could be changed into a task 
	  thetask.addr=32'h00000000; // 1st word 
	  thetask.data=32'h10010000;  // spare
	  thetask.job = WRITE_1OFN;
	  #1 -> xactn;   @done;
	  //
	  thetask.addr=thetask.addr + 4; // 2nd addr
	  thetask.data=thetask.data;  // 1st data   
	  thetask.job = WRITE_NOFM;
	  #1 -> xactn;   @done;
	  //
	  thetask.addr=thetask.addr + 4; // 3nd addr
	  thetask.data=thetask.data + 4;  // 2nd data   
	  thetask.job = WRITE_NOFM;
	  #1 -> xactn;   @done;
	  //
	  thetask.addr=thetask.addr + 4; // 4th addr
	  thetask.data=thetask.data + 4;  // 3rd data   
	  thetask.job = WRITE_NOFM;
	  #1 -> xactn;   @done;
	  //
	  thetask.addr=thetask.addr; // spare 
	  thetask.data=thetask.data + 4;  // 4th data   
	  thetask.job = WRITE_LASTOFM;
	  #1 -> xactn;   @done;
	  
	  
	   // do idle
	  thetask.job = T_IDLE;
      #1 -> xactn;   @done;

	  // do a read SEQ of 4 
	  thetask.addr=32'h00000000;
      thetask.job = READ_SEQ4;
      #1 -> xactn;   @done;

	   // do idle
	  thetask.job = T_IDLE;
      #1 -> xactn;   @done;

	  // other memory
	  // do 4 writes
	  thetask.addr=32'hFFFFF000; // 1st word 
	  thetask.data=32'hFFFFF000;  // spare
	  thetask.job = WRITE_1OFN;
	  #1 -> xactn;   @done;
	  //
	  thetask.addr=thetask.addr + 4; // 2nd addr
	  thetask.data=thetask.data;  // 1st data   
	  thetask.job = WRITE_NOFM;
	  #1 -> xactn;   @done;
	  //
	  thetask.addr=thetask.addr + 4; // 3nd addr
	  thetask.data=thetask.data + 4;  // 2nd data   
	  thetask.job = WRITE_NOFM;
	  #1 -> xactn;   @done;
	  //
	  thetask.addr=thetask.addr + 4; // 4th addr
	  thetask.data=thetask.data + 4;  // 3rd data   
	  thetask.job = WRITE_NOFM;
	  #1 -> xactn;   @done;
	  //
	  thetask.addr=thetask.addr; // spare 
	  thetask.data=thetask.data + 4;  // 4th data   
	  thetask.job = WRITE_LASTOFM;
	  #1 -> xactn;   @done;
	  
	  
	   // do idle
	  thetask.job = T_IDLE;
      #1 -> xactn;   @done;

	  // do a read SEQ of 4 
	  thetask.addr=32'hFFFFF000;
      thetask.job = READ_SEQ4;
      #1 -> xactn;   @done;

	   // do idle
	  thetask.job = T_IDLE;
      #1 -> xactn;   @done;

  	  // do a WRITE_WRAP4
	  thetask.data=32'h00010000;
	  thetask.addr=32'h00000000;
      thetask.job = WRITE_WRAP4;
      #1 -> xactn;   @done;

	  
	  //wait for a while 
	  repeat (40) @ (posedge(server.hclk));
	  $finish;
	end							// 
endmodule // client