`timescale 1 ns / 100 ps
`include "ahb_param95.v"
module tb;
  wire [31:0] hwdata;   // master Write data bus
  wire [31:0] hrdata;   // Slave Read data bus
  wire 		 hready;   // Slave Transfer done; HIGH => finished; LOW => extend transfer.
  wire [hresp_width -1:0]  hresp;    // Transfer response: OKAY; ERROR; RETRY and SPLIT.
  wire [31:0] haddr;    // Address bus; Master 
  wire [[htrans_width - 1:0]  htrans;   // Master Transfer type;  NONSEQUENTIAL; SEQUENTIAL; IDLE or BUSY.
  wire 		 hwrite;   // Transfer direction; HIGH =>  write; LOW => read 
  wire [hsize_width - 1:0]  hsize;    // Transfer size; byte (8-bit); halfword (16-bit) or word (32-bit).
  wire [hburst_width - 1 :0]  hburst;   // Burst type; Four; eight and sixteen beat; incrementing or wrapping.
  wire [3:0]  hprot;    // Protection control
  wire 		 hsel_0n;  // Slave selec
  wire 		 hresetn;  // Reset  active LOW 
  wire 		 hclk;      // Bus clock;  rising edge of HCLK
 //  wire 		 hsel;
  
  //  idt71v433s10
//  wire [31:0] idt_data; // idt71v433s10 bidirectional data
  wire [14:0] #(3) idt_addr; // idt71v433s10 address 
  wire #(3)   oe_;  // idt71v433s10 output enable active low
  wire #(3)   ce0_;   // #Zero chip idt71v433s10 chip enable, active low
  wire #(3)   ce1_;   // #One chip idt71v433s10 chip enable, active low	
  //wire #(3)		 ce_;  // idt71v433s10 chip enable; active low
  wire #(3)		 cs0;  // idt71v433s10 chip select; active high
  wire #(3)		 cs1_; // idt71v433s10 chip select; active low
  wire #(3)		 lbo_; // idt71v433s10 linear burst; low==linear count 
  wire #(3)		 gw_;  // idt71v433s10 global write 
  wire #(3)		 bwe_; // idt71v433s10 byte write enable 
  wire #(3)		 bw4_, bw_, bw2_, bw1_; // idt71v433s10 byte write 
  wire #(3)		 adsp_; // idt71v433s10 address status process; 1'b0 -> load address 
  wire #(3)		 adsc_; // idt71v433s10 address status cache; tie to 1'b1 
  wire #(3)		 adv_; // idt71v433s10 burst address advance 
  wire #(3)      wdata_enb; // 0-> hwdata to D, 1-> Z
  wire [31:0] #(3) D, Dtemp; 
  wire [14:0] #(3) A;

  localparam  number_idtmem = 0; // 0 -> full board, 1 -> 1 mem unit 
 //// sugar default clock=(posedge hclk);
 //// sugar property IdtMemReadDataOK =
 ////  always ({tb.mem_slave_1.qNonSeqReadTransfer} |=> 
 ////          {true;
 ////           tb.idt71v433s10_1.D==
 ////             {tb.idt71v433s10_1.memb4[tb.idt71v433s10_1.m_ad],
 ////              tb.idt71v433s10_1.memb3[tb.idt71v433s10_1.m_ad],
 ////              tb.idt71v433s10_1.memb2[tb.idt71v433s10_1.m_ad],
 ////              tb.idt71v433s10_1.memb1[tb.idt71v433s10_1.m_ad]} });

 //// sugar assert IdtMemReadDataOK;

  // tri-state connection to data memory         --------
  assign 	  D =  wdata_enb==1'b0 ? hwdata : 32'hzzzzzzzz; 
  assign 	  hprot = 4'h0;
  // emulate a pullup bus 
  assign      Dtemp =  (D===32'hzzzzzzzz) ? 32'h11111111 : D;
  

  mem_slave 
	#(.number_idtmem (number_idtmem)) // 0 -> full board, 1 -> 1 mem unit
    mem_slave_1
	( // output
   .hrdata (hrdata[31:0]),     // Slave Read data bus
   .hready (hready),    // Slave Transfer done, HIGH => finished; LOW => extend transfer.
   .hresp  (hresp [hresp_width - 1:0]),    // Transfer response: OKAY, ERROR, RETRY and SPLIT.
    // input
  .hwdata   (hwdata[31:0] ),   // master Write data bus  // <<< not needed ? 
  .haddr    (haddr [31:0] ),    // Address bus  (), Master 
  .htrans   (htrans[htrans_width - 1 :0]),   // Master Transfer type,  NONSEQUENTIAL, SEQUENTIAL, IDLE or BUSY.
  .hwrite   (hwrite ),   // Transfer direction, HIGH =>  write; LOW => read 
  .hsize    (hsize[hsize_width - 1:0]),    // Transfer size, byte (8-bit), halfword (16-bit) or word (32-bit).
  .hburst   (hburst [hburst_width - 1 :0]),   // Burst type, Four, eight and sixteen beat, incrementing or wrapping.
  .hprot    (hprot  [3:0]),    // Protection control <<-- not needed ?? 
  .hsel_0n  (hsel_0n),  // Slave selec
  .hresetn  (hresetn),  // Reset  active LOW 
  .hclk     (hclk   ),      // Bus clock,  rising edge of HCLK
  //  idt71v433s10
  .wdata_enb (wdata_enb),  // 0-> write hwdata to memory, 1-> Z 
  .idt_data (Dtemp[31:0]), // idt71v433s10 bidirectional data after pullup
  .idt_addr (idt_addr[14:0]), // idt71v433s10 address 
  // output
  .oe_   (oe_  ),  // idt71v433s10 output enable, active low  
  .ce0_   (ce0_  ),  // idt71v433s10 chip enable, active low
  .ce1_   (ce1_  ),  // idt71v433s10 chip enable, active low	  
  .cs0   (cs0  ),  // idt71v433s10 chip select, active high
  .cs1_  (cs1_ ), // idt71v433s10 chip select, active low
  .lbo_  (lbo_ ), // idt71v433s10 linear burst, low==linear count 
  .gw_   (gw_  ),  // idt71v433s10 global write 
  .bwe_  (bwe_ ), // idt71v433s10 byte write enable 
  .bw4_  (bw4_ ), 
  .bw3_  (bw3_ ), 
  .bw2_  (bw2_ ), 
  .bw1_  (bw1_ ), // idt71v433s10 byte write 
  .adsp_ (adsp_), // idt71v433s10 address status process, 1'b0 -> load address 
  .adsc_ (adsc_), // idt71v433s10 address status cache, tie to 1'b1 
  .adv_  (adv_ ) // idt71v433s10 burst address advance 
 );

  
  server server
	(
	 // Inputs
	 .hrdata    (hrdata[31:0]), // Slave Read data bus                                            
	 .hready    (hready),		// Slave Transfer done, HIGH => finished; LOW => extend transfer. 
	 .hresp		(hresp[hresp_width-1:0]),		// Transfer response: OKAY, ERROR, RETRY and SPLIT.               
     // outputs 
     .hwdata  (hwdata[31:0] ),   // master Write data bus                                        
     .haddr   (haddr[31:0]  ),    // Address bus, Master 
     .htrans  (htrans[htrans_width-1:0] ),   // Master Transfer type,  NONSEQUENTIAL, SEQUENTIAL, IDLE or BUSY.
     .hwrite  (hwrite ),   // Transfer direction, HIGH =>  write; LOW => read 
     .hsize   (hsize [hsize_width-1:0] ),    // Transfer size, byte (8-bit), halfword (16-bit) or word (32-bit).
     .hburst  (hburst[hburst_width-1:0] ),   // Burst type, Four, eight and sixteen beat, incrementing or wrapping.
     .hprot   (hprot [3:0] ),    // Protection control
     .hsel_sn (hsel_0n),  // Slave selec
     .hresetn (hresetn),  // Reset  active LOW 
     .hclk    (hclk   )  // Bus clock,  rising edge of HCLK
	 );
  
  
    // 1st memory unit 
    idt71v433s10   idt71v433s10_1 (
								 // Inouts
								 .D		(D[31:0]),
								 // Inputs
								 .A		(idt_addr[14:0]),
								 .oe_	(oe_),
								 .ce_	(ce0_),
								 .cs0	(cs0),
								 .cs1_	(cs1_),
								 .lbo_	(lbo_),
								 .gw_	(gw_),
								 .bwe_	(bwe_),
								 .bw4_	(bw4_),
								 .bw3_	(bw3_),
								 .bw2_	(bw2_),
								 .bw1_	(bw1_),
								 .adsp_	(adsp_),
								 .adsc_	(adsc_),
								 .adv_	(adv_),
								 .clk	(hclk));

   // 2nd memory unit  // 0 -> full board, 1 -> 1 mem unit
  // <<< generate COMPILES ok IN mti
//  generate 
//  if (number_idtmem == 0) 
	idt71v433s10  idt71v433s10_2 (
								 // Inouts
								 .D		(D[31:0]),
								 // Inputs
								 .A		(idt_addr[14:0]),
								 .oe_	(oe_),
								 .ce_	(ce1_),
								 .cs0	(cs0),
								 .cs1_	(cs1_),
								 .lbo_	(lbo_),
								 .gw_	(gw_),
								 .bwe_	(bwe_),
								 .bw4_	(bw4_),
								 .bw3_	(bw3_),
								 .bw2_	(bw2_),
								 .bw1_	(bw1_),
								 .adsp_	(adsp_),
								 .adsc_	(adsc_),
								 .adv_	(adv_),
								 .clk	(hclk));
  // endgenerate 

  
  thetask thetask ();
  
  client client();

  ahbCompliance ahbCompliance
    (
	 // inputs
	 .hclk		(hclk),
	 .hresetn	(hresetn),
	 .haddr	(haddr[31:0]),	 
	 .htrans	(htrans[1:0]),
	 .hwrite	(hwrite),
	 .hsize	(hsize[2:0]),
	 .hburst	(hburst[2:0]),	 
	 .hprot	(hprot[3:0]),
	 .hwdata	(hwdata[31:0]),	 
	 .hsel	(hsel_0n),
	 .hrdata	(hrdata[31:0]),	 
	 .hready	(hready),
	 .hresp	(hresp[1:0]));
  
  
  
  
  
endmodule
